@@ -5,13 +5,14 @@
 #include <WINGs/WINGs.h>
 
 #include <stdio.h>
-
-
+#include <stdlib.h>
+#include <string.h>
 
 /*
  * You need to define this function to link any program to WINGs.
+ * (this is no longer required as there is a default abort handler in WINGs)
  * This will be called when the application will be terminated because
- * on a fatal error.
+ * of a fatal error (only for memory allocation failures ATM).
  */
 void
 wAbort()
@@ -33,7 +34,7 @@ closeAction(WMWidget *self, void *data)
     windowCount--;
     printf("window closed, window count = %d\n", windowCount);
     if (windowCount < 1)
-	exit(0);
+        exit(0);
 }
 
 
@@ -65,7 +66,7 @@ testFontPanel(WMScreen *scr)
 
     WMShowFontPanel(panel);
 
-/*    WMFreeFontPanel(panel);*/
+    /*WMFreeFontPanel(panel);*/
 }
 
 
@@ -77,15 +78,15 @@ testFrame(WMScreen *scr)
     WMFrame *frame;
     int i;
     static char* titles[] = {
-	"AboveTop",
-	    "AtTop",
-	    "BelowTop",
-	    "AboveBottom",
-	    "AtBottom",
-	    "BelowBottom"
+        "AboveTop",
+        "AtTop",
+        "BelowTop",
+        "AboveBottom",
+        "AtBottom",
+        "BelowBottom"
     };
     static WMTitlePosition pos[] = {
-	WTPAboveTop,
+        WTPAboveTop,
         WTPAtTop,
         WTPBelowTop,
         WTPAboveBottom,
@@ -101,11 +102,11 @@ testFrame(WMScreen *scr)
     WMResizeWidget(win, 400, 300);
 
     for (i = 0; i < 6; i++) {
-	frame = WMCreateFrame(win);
-	WMMoveWidget(frame, 8+(i%3)*130, 8+(i/3)*130);
-	WMResizeWidget(frame, 120, 120);
-	WMSetFrameTitle(frame, titles[i]);
-	WMSetFrameTitlePosition(frame, pos[i]);
+        frame = WMCreateFrame(win);
+        WMMoveWidget(frame, 8+(i%3)*130, 8+(i/3)*130);
+        WMResizeWidget(frame, 120, 120);
+        WMSetFrameTitle(frame, titles[i]);
+        WMSetFrameTitlePosition(frame, pos[i]);
     }
 
     WMRealizeWidget(win);
@@ -115,14 +116,14 @@ testFrame(WMScreen *scr)
 }
 
 
-static void
-resizedWindow(void *self, WMNotification *notif)
-{
-    WMView *view = (WMView*)WMGetNotificationObject(notif);
-    WMSize size = WMGetViewSize(view);
+/*static void
+ resizedWindow(void *self, WMNotification *notif)
+ {
+ WMView *view = (WMView*)WMGetNotificationObject(notif);
+ WMSize size = WMGetViewSize(view);
 
-    WMResizeWidget((WMWidget*)self, size.width, size.height);
-}
+ WMResizeWidget((WMWidget*)self, size.width, size.height);
+ }*/
 
 void
 testBox(WMScreen *scr)
@@ -138,45 +139,39 @@ testBox(WMScreen *scr)
     win = WMCreateWindow(scr, "testBox");
     WMSetWindowTitle(win, "Box");
     WMSetWindowCloseAction(win, closeAction, NULL);
-
-    WMSetViewNotifySizeChanges(WMWidgetView(win), True);
+    WMResizeWidget(win, 400, 300);
 
     box = WMCreateBox(win);
     WMSetBoxBorderWidth(box, 5);
-    
-    WMAddNotificationObserver(resizedWindow, box,
-			      WMViewSizeDidChangeNotification, 
-			      WMWidgetView(win));
-    WMResizeWidget(win, 400, 300);
-    
+    WMSetViewExpandsToParent(WMWidgetView(box), 0, 0, 0, 0);
 
-/*    WMSetBoxHorizontal(box, True); */
+    /*WMSetBoxHorizontal(box, True);*/
     for (i = 0; i < 4; i++) {
-	btn = WMCreateCommandButton(box);
-	WMSetButtonText(btn, "bla");
-	WMMapWidget(btn);
-	WMAddBoxSubview(box, WMWidgetView(btn), i&1, True, 20, 0, 5);
+        btn = WMCreateCommandButton(box);
+        WMSetButtonText(btn, "bla");
+        WMMapWidget(btn);
+        WMAddBoxSubview(box, WMWidgetView(btn), i&1, True, 20, 0, 5);
     }
-    
+
     pop = WMCreatePopUpButton(box);
     WMAddPopUpButtonItem(pop, "ewqeq");
     WMAddPopUpButtonItem(pop, "ewqeqrewrw");
     WMAddBoxSubview(box, WMWidgetView(pop), False, True, 20, 0, 5);
     WMMapWidget(pop);
-    
+
     hbox = WMCreateBox(box);
     WMSetBoxHorizontal(hbox, True);
     WMAddBoxSubview(box, WMWidgetView(hbox), False, True, 24, 0, 0);
     WMMapWidget(hbox);
- 
+
     for (i = 0; i < 4; i++) {
-	btn = WMCreateCommandButton(hbox);
-	WMSetButtonText(btn, "bla");
-	WMMapWidget(btn);
-	WMAddBoxSubview(hbox, WMWidgetView(btn), 1, True, 60, 0, i<3?5:0);
+        btn = WMCreateCommandButton(hbox);
+        WMSetButtonText(btn, "bla");
+        WMMapWidget(btn);
+        WMAddBoxSubview(hbox, WMWidgetView(btn), 1, True, 60, 0, i<3?5:0);
     }
-     
-    
+
+
     WMRealizeWidget(win);
     WMMapSubwidgets(win);
     WMMapWidget(win);
@@ -246,17 +241,17 @@ testList(WMScreen *scr)
     list = WMCreateList(win);
     /*WMSetListAllowEmptySelection(list, True);*/
     WMMoveWidget(list, 10, 40);
-    for (i=0; i<50; i++) {
-	sprintf(text, "Item %i", i);
-	WMAddListItem(list, text);
+    for (i=0; i<105; i++) {
+        sprintf(text, "Item %i", i);
+        WMAddListItem(list, text);
     }
     mlist = WMCreateList(win);
     WMSetListAllowMultipleSelection(mlist, True);
     /*WMSetListAllowEmptySelection(mlist, True);*/
     WMMoveWidget(mlist, 210, 40);
     for (i=0; i<135; i++) {
-	sprintf(text, "Item %i", i);
-	WMAddListItem(mlist, text);
+        sprintf(text, "Item %i", i);
+        WMAddListItem(mlist, text);
     }
 
     label = WMCreateLabel(win);
@@ -277,9 +272,9 @@ testList(WMScreen *scr)
     WMSetListDoubleAction(mlist, doubleClick, mlabel);
 
     WMAddNotificationObserver(listSelectionObserver, label,
-			      WMListSelectionDidChangeNotification, list);
+                              WMListSelectionDidChangeNotification, list);
     WMAddNotificationObserver(listSelectionObserver, mlabel,
-			      WMListSelectionDidChangeNotification, mlist);
+                              WMListSelectionDidChangeNotification, mlist);
 
 
     WMRealizeWidget(win);
@@ -294,14 +289,14 @@ testButton(WMScreen *scr)
     WMWindow *win;
     int i;
     char *types[] = {
-	"MomentaryPush",
-	    "PushOnPushOff",
-	    "Toggle",
-	    "Switch",
-	    "Radio",
-	    "MomentaryChange",
-	    "OnOff",
-	    "MomentaryLigh"
+        "MomentaryPush",
+        "PushOnPushOff",
+        "Toggle",
+        "Switch",
+        "Radio",
+        "MomentaryChange",
+        "OnOff",
+        "MomentaryLigh"
     };
 
     windowCount++;
@@ -313,11 +308,11 @@ testButton(WMScreen *scr)
     WMSetWindowCloseAction(win, closeAction, NULL);
 
     for (i = 1; i < 9; i++) {
-	WMButton *b;
-	b = WMCreateButton(win, i);
-	WMResizeWidget(b, 150, 24);
-	WMMoveWidget(b, 20, i*30);
-	WMSetButtonText(b, types[i-1]);
+        WMButton *b;
+        b = WMCreateButton(win, i);
+        WMResizeWidget(b, 150, 24);
+        WMMoveWidget(b, 20, i*30);
+        WMSetButtonText(b, types[i-1]);
     }
 
     WMRealizeWidget(win);
@@ -343,6 +338,9 @@ testGradientButtons(WMScreen *scr)
     WMSetWindowTitle(win, "Gradiented Button Demo");
     WMResizeWidget(win, 300, 200);
 
+    WMSetWindowCloseAction(win, closeAction, NULL);
+
+
     light.red = 0x90;
     light.green = 0x85;
     light.blue = 0x90;
@@ -406,7 +404,7 @@ testGradientButtons(WMScreen *scr)
     WMSetButtonTextColor(btn, color);
 
     WMSetBalloonTextForView("This is yet another button.\nBut the balloon has 3 lines.\nYay!",
-			    WMWidgetView(btn));
+                            WMWidgetView(btn));
 
     WMReleaseColor(color);
     WMReleaseColor(altColor);
@@ -452,12 +450,12 @@ testScrollView(WMScreen *scr)
     WMSetFrameRelief(f, WRFlat);
 
     for (i=0; i<20; i++) {
-	l = WMCreateLabel(f);
-	WMResizeWidget(l, 50, 18);
-	WMMoveWidget(l, 10, 20*i);
-	sprintf(buffer, "Label %i", i);
-	WMSetLabelText(l, buffer);
-	WMSetLabelRelief(l, WRSimple);
+        l = WMCreateLabel(f);
+        WMResizeWidget(l, 50, 18);
+        WMMoveWidget(l, 10, 20*i);
+        sprintf(buffer, "Label %i", i);
+        WMSetLabelText(l, buffer);
+        WMSetLabelRelief(l, WRSimple);
     }
     WMMapSubwidgets(f);
     WMMapWidget(f);
@@ -511,10 +509,10 @@ testColorPanel(WMScreen *scr)
     WMColorPanel *panel = WMGetColorPanel(scr);
 
     /*if (colorname) {
-        startcolor = WMCreateNamedColor(scr, colorname, False);
-        WMSetColorPanelColor(panel, startcolor);
-        WMReleaseColor(startcolor);
-    }*/
+     startcolor = WMCreateNamedColor(scr, colorname, False);
+     WMSetColorPanelColor(panel, startcolor);
+     WMReleaseColor(startcolor);
+     }*/
 
     WMShowColorPanel(panel);
 }
@@ -592,7 +590,9 @@ testText(WMScreen *scr)
 {
     WMWindow *win;
     WMText *text;
-    FILE *file = fopen("wm.html", "r");
+    WMFont *font;
+    void *tb;
+    FILE *file = fopen("wm.html", "rb");
 
     windowCount++;
 
@@ -606,8 +606,22 @@ testText(WMScreen *scr)
     WMMoveWidget(text, 10, 10);
     WMSetTextHasVerticalScroller(text, True);
     WMSetTextEditable(text, False);
+    WMSetTextIgnoresNewline(text, False);
 
-    if(file) {
+#define FNAME "Verdana,Luxi Sans:pixelsize=12"
+#define MSG \
+    "Window Maker is the GNU window manager for the " \
+    "X Window System. It was designed to emulate the " \
+    "look and feel of part of the NEXTSTEP(tm) GUI. It's " \
+    "supposed to be relatively fast and small, feature " \
+    "rich, easy to configure and easy to use, with a simple " \
+    "and elegant appearance borrowed from NEXTSTEP(tm)."
+
+    font = WMCreateFont(scr, FNAME":autohint=false");
+    WMSetTextDefaultFont(text, font);
+    WMReleaseFont(font);
+
+    if (0 && file) {
         char buf[1024];
 
         WMFreezeText(text);
@@ -617,13 +631,16 @@ testText(WMScreen *scr)
         fclose(file);
         WMThawText(text);
     } else {
-        WMAppendTextStream(text,
-                           "Window Maker is the GNU window manager for the "
-                           "X Window System. It was designed to emulate the "
-                           "look and feel of part of the NEXTSTEP(tm) GUI. Its "
-                           "supposed to be relatively fast and small, feature "
-                           "rich, easy to configure and easy to use, with a simple "
-                           "and elegant appearance borrowed from NEXTSTEP(tm).");
+        WMAppendTextStream(text, "First paragraph has autohinting turned off, "
+                           "while the second has it turned on:");
+        WMAppendTextStream(text, "\n\n\n");
+        WMAppendTextStream(text, MSG);
+        WMAppendTextStream(text, "\n\n\n");
+        font = WMCreateFont(scr, FNAME":autohint=true");
+        tb = WMCreateTextBlockWithText(text, MSG, font, WMBlackColor(scr),
+                                       0, strlen(MSG));
+        WMAppendTextBlock(text, tb);
+        WMReleaseFont(font);
     }
 
     WMRealizeWidget(win);
@@ -792,21 +809,21 @@ testTabView(WMScreen *scr)
 
 void
 splitViewConstrainProc(WMSplitView *sPtr, int indView,
-		       int *minSize, int *maxSize)
+                       int *minSize, int *maxSize)
 {
     switch (indView) {
     case 0:
-    	*minSize = 20;
-	break;
+        *minSize = 20;
+        break;
     case 1:
-    	*minSize = 40;
-    	*maxSize = 80;
-	break;
+        *minSize = 40;
+        *maxSize = 80;
+        break;
     case 2:
-    	*maxSize = 60;
-	break;
+        *maxSize = 60;
+        break;
     default:
-    	break;
+        break;
     }
 }
 
@@ -817,14 +834,14 @@ resizeSplitView(XEvent *event, void *data)
     WMSplitView *sPtr = (WMSplitView*)data;
 
     if (event->type == ConfigureNotify) {
-	int width = event->xconfigure.width - 10;
+        int width = event->xconfigure.width - 10;
 
-	if (width < WMGetSplitViewDividerThickness(sPtr))
-	    width = WMGetSplitViewDividerThickness(sPtr);
+        if (width < WMGetSplitViewDividerThickness(sPtr))
+            width = WMGetSplitViewDividerThickness(sPtr);
 
-	if (width != WMWidgetWidth(sPtr) ||
+        if (width != WMWidgetWidth(sPtr) ||
             event->xconfigure.height != WMWidgetHeight(sPtr))
-	    WMResizeWidget(sPtr, width, event->xconfigure.height - 55);
+            WMResizeWidget(sPtr, width, event->xconfigure.height - 55);
     }
 }
 
@@ -850,9 +867,9 @@ removeSubviewButtonAction(WMWidget *self, void *data)
     int count = WMGetSplitViewSubviewsCount(sPtr);
 
     if (count > 2) {
-    	WMView *view = WMGetSplitViewSubviewAt(sPtr, count-1);
-	WMDestroyWidget(WMWidgetOfView(view));
-    	WMRemoveSplitViewSubviewAt(sPtr, count-1);
+        WMView *view = WMGetSplitViewSubviewAt(sPtr, count-1);
+        WMDestroyWidget(WMWidgetOfView(view));
+        WMRemoveSplitViewSubviewAt(sPtr, count-1);
     }
 }
 
@@ -957,275 +974,6 @@ testSplitView(WMScreen *scr)
 }
 
 
-/*******************************************************************/
-
-#include <sys/types.h>
-#include <dirent.h>
-#include <string.h>
-
-
-typedef struct {
-    int x, y;
-    Bool mouseDown;
-    char *filename;
-} DNDStuff;
-
-WMPixmap*
-getImage(WMScreen *scr, char *file)
-{
-    char buffer[1000];
-    WMPixmap *pix;
-
-    sprintf(buffer, "../../WindowMaker/Icons/%s", file);
-    pix = WMCreatePixmapFromFile(scr, buffer);
-
-    return pix;
-}
-
-
-
-
-static void
-iconMouseStuff(XEvent *event, void *cdata)
-{
-    WMLabel *label = (WMLabel*)cdata;
-    DNDStuff *stuff = WMGetHangedData(label);
-    WMPoint where;
-
-    switch (event->type) {
-     case ButtonPress:
-	stuff->x = event->xbutton.x;
-	stuff->y = event->xbutton.y;
-	stuff->mouseDown = True;
-	break;
-     case ButtonRelease:
-	stuff->mouseDown = False;
-	break;
-     case MotionNotify:
-	if (!stuff->mouseDown)
-	    break;
-
-	if (abs(stuff->x - event->xmotion.x)>4
-	    || abs(stuff->y - event->xmotion.y)>4) {
-
-	    where = WMGetViewScreenPosition(WMWidgetView(label));
-
-	    WMDragImageFromView(WMWidgetView(label),
-				WMGetLabelImage(label),
-				NULL, /* XXX */
-				where,
-				wmksize(stuff->x, stuff->y),
-				event, True);
-	}
-	break;
-    }
-}
-
-
-static void
-endedDragImage(WMView *self, WMPixmap *image, WMPoint point, Bool deposited)
-{
-    DNDStuff *stuff = WMGetHangedData(WMWidgetOfView(self));
-
-    if (deposited) {
-	WMDestroyWidget(WMWidgetOfView(self));
-    }
-
-    stuff->mouseDown = False;
-}
-
-
-static WMData*
-fetchDragData(WMView *self, char *type)
-{
-    DNDStuff *stuff = WMGetHangedData(WMWidgetOfView(self));
-
-    return WMCreateDataWithBytes(stuff->filename, strlen(stuff->filename)+1);
-}
-
-
-WMDragSourceProcs dragSourceProcs = {
-    NULL,
-    NULL,
-    endedDragImage,
-    fetchDragData
-};
-
-
-/************************/
-
-
-unsigned
-draggingEntered(WMView *self, WMDraggingInfo *info)
-{
-    return WDOperationCopy;
-}
-
-
-unsigned
-draggingUpdated(WMView *self, WMDraggingInfo *info)
-{
-    return WDOperationCopy;
-}
-
-/*
- void (*draggingExited)(WMView *self, WMDraggingInfo *info);
- */
-char*
-prepareForDragOperation(WMView *self, WMDraggingInfo *info)
-{
-    return "application/X-WINGs-Bla";
-}
-
-
-WMLabel* makeDraggableLabel(WMWidget *w, char *file, int x, int y);
-
-Bool
-performDragOperation(WMView *self, WMDraggingInfo *info, WMData *data)
-{
-    char *file = (char*)WMDataBytes(data);
-    WMPoint pos;
-
-    pos = WMGetDraggingInfoImageLocation(info);
-
-    if (file!=NULL) {
-	WMLabel *label;
-	WMPoint pos2 = WMGetViewScreenPosition(self);
-
-
-	label = makeDraggableLabel(WMWidgetOfView(self), file,
-				   pos.x-pos2.x, pos.y-pos2.y);
-	WMRealizeWidget(label);
-	WMMapWidget(label);
-    }
-
-
-    return True;
-}
-
-
-void
-concludeDragOperation(WMView *self, WMDraggingInfo *info)
-{
-
-}
-
-
-
-WMDragDestinationProcs dragDestProcs = {
-    draggingEntered,
-	draggingUpdated,
-	NULL,
-	prepareForDragOperation,
-	performDragOperation,
-	concludeDragOperation
-};
-
-
-
-
-WMLabel*
-makeDraggableLabel(WMWidget *w, char *file, int x, int y)
-{
-    DNDStuff *stuff;
-    WMLabel *label;
-    WMPixmap *image = getImage(WMWidgetScreen(w), file);
-
-    stuff = wmalloc(sizeof(DNDStuff));
-    stuff->mouseDown = False;
-
-    stuff->filename = wstrdup(file);
-
-    label = WMCreateLabel(w);
-    WMResizeWidget(label, 48, 48);
-    WMMoveWidget(label, x, y);
-
-    WMSetViewDragSourceProcs(WMWidgetView(label), &dragSourceProcs);
-
-    WMHangData(label, stuff);
-
-    WMCreateEventHandler(WMWidgetView(label),
-			 ButtonPressMask|ButtonReleaseMask|ButtonMotionMask,
-			 iconMouseStuff, label);
-
-
-    if (image != NULL) {
-	WMSetLabelImagePosition(label, WIPImageOnly);
-	WMSetLabelImage(label, image);
-	WMReleasePixmap(image);
-    } else puts(file);
-
-    return label;
-}
-
-
-
-void
-testDragAndDrop(WMScreen *scr)
-{
-    WMWindow *win;
-    WMFrame *frame;
-    WMLabel *label;
-    int i, j;
-    DIR *dir;
-    struct dirent *ent;
-    char *types[] = {
-	"application/X-WINGs-Bla",
-        NULL
-    };
-
-    windowCount++;
-
-    win = WMCreateWindow(scr, "dragDrop");
-    WMResizeWidget(win, 300, 300);
-    WMSetWindowCloseAction(win, closeAction, NULL);
-    WMSetWindowTitle(win, "Drag and Drop");
-
-
-    frame = WMCreateFrame(win);
-    WMSetFrameRelief(frame, WRSunken);
-    WMResizeWidget(frame, 250, 250);
-    WMMoveWidget(frame, 25, 25);
-
-    WMRegisterViewForDraggedTypes(WMWidgetView(frame), types);
-    WMSetViewDragDestinationProcs(WMWidgetView(frame), &dragDestProcs);
-
-    dir = opendir("../../WindowMaker/Icons");
-    if (!dir) {
-	perror("../../WindowMaker/Icons");
-	return;
-    }
-
-    for (i = 0, j=0; j < 8; i++) {
-	ent = readdir(dir);
-	if (!ent)
-	    break;
-
-	if (strstr(ent->d_name, ".xpm")==NULL) {
-	    continue;
-	}
-
-	label = makeDraggableLabel(frame, ent->d_name,4+(j/4)*64, 4+(j%4)*64);
-
-	j++;
-    }
-
-    closedir(dir);
-
-    WMMapSubwidgets(frame);
-
-    WMMapSubwidgets(win);
-    WMRealizeWidget(win);
-    WMMapWidget(win);
-}
-
-
-
-
-
-/*******************************************************************/
-
-
 void
 testUD()
 {
@@ -1259,15 +1007,15 @@ main(int argc, char **argv)
     dpy = XOpenDisplay("");
 
     if (!dpy) {
-	puts("could not open display");
-	exit(1);
+        puts("could not open display");
+        exit(1);
     }
 
     /* This is used to disable buffering of X protocol requests.
      * Do NOT use it unless when debugging. It will cause a major
      * slowdown in your application
      */
-#if 1
+#if 0
     XSynchronize(dpy, True);
 #endif
     /*
@@ -1283,7 +1031,10 @@ main(int argc, char **argv)
     /*
      * Makes the logo be used in standard dialog panels.
      */
-    WMSetApplicationIconPixmap(scr, pixmap); WMReleasePixmap(pixmap);
+    if (pixmap) {
+        WMSetApplicationIconPixmap(scr, pixmap);
+        WMReleasePixmap(pixmap);
+    }
 
     /*
      * Do some test stuff.
@@ -1291,44 +1042,29 @@ main(int argc, char **argv)
      * Put the testSomething() function you want to test here.
      */
 
-
-
-    testDragAndDrop(scr);
     testText(scr);
-#if 0
-    testColorPanel(scr);
-    testScrollView(scr);
-    testTabView(scr);    
-    testBox(scr);
-    testText(scr);
-    testList(scr);
-
-    testProgressIndicator(scr);
-
-    testColorWell(scr);
-
-    testTextField(scr);
-
-    testDragAndDrop(scr);
     testFontPanel(scr);
 
+    testColorPanel(scr);
+
+#if 0
+
+    testBox(scr);
     testButton(scr);
-
+    testColorPanel(scr);
+    testColorWell(scr);
+    testDragAndDrop(scr);
     testFrame(scr);
-
-
-
-
-
-    testSplitView(scr);
-
     testGradientButtons(scr);
-
-
+    testList(scr);
     testOpenFilePanel(scr);
-
-    testSlider(scr);
+    testProgressIndicator(scr);
     testPullDown(scr);
+    testScrollView(scr);
+    testSlider(scr);
+    testSplitView(scr);
+    testTabView(scr);
+    testTextField(scr);
 #endif
     /*
      * The main event loop.
@@ -1338,3 +1074,4 @@ main(int argc, char **argv)
 
     return 0;
 }
+
