@@ -1,8 +1,9 @@
 /*
  *  Window Maker window manager
- * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- * 
+ *
+ *  Copyright (c) 1997-2003 Alfredo K. Kojima
+ *  Copyright (c) 1998-2003 Dan Pascu
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -15,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
  *  USA.
  */
 #include "wconfig.h"
@@ -24,11 +25,18 @@
 
 #include <stdlib.h>
 #include <string.h>
+
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "WindowMaker.h"
 #include "menu.h"
 #include "window.h"
+#ifdef USER_MENU
+#include "usermenu.h"
+#endif /* USER_MENU */
 #include "icon.h"
 #include "appicon.h"
 #include "application.h"
@@ -39,12 +47,10 @@
 #include "actions.h"
 #include "defaults.h"
 #include "workspace.h"
-
 #include "dock.h"
+#include "wsound.h"
 
-#ifdef WMSOUND
-#include "wmsound.h"
-#endif
+#include "xinerama.h"
 
 
 /******** Global variables ********/
@@ -64,9 +70,8 @@ makeMainWindow(WScreen *scr, Window window)
     WWindow *wwin;
     XWindowAttributes attr;
 
-    
     if (!XGetWindowAttributes(dpy, window, &attr)) {
-	return NULL;
+        return NULL;
     }
 
     wwin = wWindowCreate();
@@ -74,17 +79,18 @@ makeMainWindow(WScreen *scr, Window window)
     wwin->client_win = window;
     wwin->main_window = window;
     wwin->wm_hints = XGetWMHints(dpy, window);
-/*    if (!MyXFetchName(dpy, window, &(wwin->frame->title))) {
-	wwin->frame->title = NULL;
-    }
- */
+    /*    if (!MyXFetchName(dpy, window, &(wwin->frame->title))) {
+     wwin->frame->title = NULL;
+     }
+     */
     PropGetWMClass(window, &wwin->wm_class, &wwin->wm_instance);
 
+
     wDefaultFillAttributes(scr, wwin->wm_instance, wwin->wm_class,
-			   &wwin->window_flags, True);
-    
+                           &wwin->user_flags, &wwin->defined_user_flags, True);
+
     XSelectInput(dpy, window, attr.your_event_mask | PropertyChangeMask
-		 | StructureNotifyMask );
+                 | StructureNotifyMask);
     return wwin;
 }
 
@@ -94,10 +100,10 @@ wApplicationOf(Window window)
 {
     WApplication *wapp;
 
-    if (window == None) 
-      return NULL;
+    if (window == None)
+        return NULL;
     if (XFindContext(dpy, window, wAppWinContext, (XPointer*)&wapp)!=XCSUCCESS)
-      return NULL;
+        return NULL;
     return wapp;
 }
 
@@ -107,10 +113,10 @@ findDockIconFor(WDock *dock, Window main_window)
 {
     WAppIcon *aicon = NULL;
 
-    aicon = wDockFindIconFor(dock, main_window);
+    aicon = wDockFindIconForWindow(dock, main_window);
     if (!aicon) {
         wDockTrackWindowLaunch(dock, main_window);
-        aicon = wDockFindIconFor(dock, main_window);
+        aicon = wDockFindIconForWindow(dock, main_window);
     }
     return aicon;
 }
@@ -119,105 +125,105 @@ findDockIconFor(WDock *dock, Window main_window)
 static void
 extractIcon(WWindow *wwin)
 {
-    int argc;
-    char **argv;
-    
-    if (!XGetCommand(dpy, wwin->client_win, &argv, &argc) || argc < 1)
-	return;
+    char *progname;
 
-    wApplicationExtractDirPackIcon(wwin->screen_ptr,argv[0], 
-				   wwin->wm_instance,
-				   wwin->wm_class);
-    XFreeStringList(argv);
+    progname = GetProgramNameForWindow(wwin->client_win);
+    if (progname) {
+        wApplicationExtractDirPackIcon(wwin->screen_ptr, progname,
+                                       wwin->wm_instance, wwin->wm_class);
+        wfree(progname);
+    }
 }
 
 
 static void
 saveIconNameFor(char *iconPath, char *wm_instance, char *wm_class)
 {
-    proplist_t dict = WDWindowAttributes->dictionary;
-    proplist_t adict, key, iconk;
-    proplist_t val;
+    WMPropList *dict = WDWindowAttributes->dictionary;
+    WMPropList *adict, *key, *iconk;
+    WMPropList *val;
     char *tmp;
     int i;
-	
+
     i = 0;
     if (wm_instance)
-	i += strlen(wm_instance);
+        i += strlen(wm_instance);
     if (wm_class)
-	i += strlen(wm_class);
+        i += strlen(wm_class);
 
     tmp = wmalloc(i+8);
     *tmp = 0;
     if (wm_class && wm_instance) {
-	sprintf(tmp, "%s.%s", wm_instance, wm_class);
+        sprintf(tmp, "%s.%s", wm_instance, wm_class);
     } else {
-	if (wm_instance)
-	    strcat(tmp, wm_instance);
-	if (wm_class)
-	    strcat(tmp, wm_class);
+        if (wm_instance)
+            strcat(tmp, wm_instance);
+        if (wm_class)
+            strcat(tmp, wm_class);
     }
-    
-    key = PLMakeString(tmp); 
-    free(tmp);
-    adict = PLGetDictionaryEntry(dict, key);
-    
-    iconk = PLMakeString("Icon");
-    
+
+    key = WMCreatePLString(tmp);
+    wfree(tmp);
+    adict = WMGetFromPLDictionary(dict, key);
+
+    iconk = WMCreatePLString("Icon");
+
     if (adict) {
-	val = PLGetDictionaryEntry(adict, iconk);
+        val = WMGetFromPLDictionary(adict, iconk);
     } else {
-	/* no dictionary for app, so create one */
-	adict = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
-	PLInsertDictionaryEntry(dict, key, adict);
-	PLRelease(adict);
-	val = NULL;
+        /* no dictionary for app, so create one */
+        adict = WMCreatePLDictionary(NULL, NULL);
+        WMPutInPLDictionary(dict, key, adict);
+        WMReleasePropList(adict);
+        val = NULL;
     }
     if (!val) {
-	val = PLMakeString(iconPath);
-	PLInsertDictionaryEntry(adict, iconk, val);
-	PLRelease(val);
+        val = WMCreatePLString(iconPath);
+        WMPutInPLDictionary(adict, iconk, val);
+        WMReleasePropList(val);
     }
-    PLRelease(key);
+    WMReleasePropList(key);
+    WMReleasePropList(iconk);
 
-    if (val && !wPreferences.flags.noupdates)
-	PLSave(dict, YES);
+    if (val && !wPreferences.flags.noupdates) {
+        UpdateDomainFile(WDWindowAttributes);
+    }
 }
 
 
 void
-wApplicationExtractDirPackIcon(WScreen *scr, char *path, 
-				char *wm_instance, char *wm_class)
+wApplicationExtractDirPackIcon(WScreen *scr, char *path,
+                               char *wm_instance, char *wm_class)
 {
     char *iconPath=NULL;
     /* Maybe the app is a .app and it has an icon in it, like
-     * /usr/local/GNUstep/Apps/WPrefs.app/WPrefs.tiff
+     * /usr/local/GNUstep/Applications/WPrefs.app/WPrefs.tiff
      */
     if (strstr(path, ".app")) {
-	char *tmp;
-	
-	tmp = wmalloc(strlen(path)+16);
-	
-	if (scr->flags.supports_tiff) {
-	    strcpy(tmp, path);
-	    strcat(tmp, ".tiff");
-	    if (access(tmp, R_OK)==0)
-		iconPath = tmp;
-	}
-	if (!path) {
-	    strcpy(tmp, path);
-	    strcat(tmp, ".xpm");
-	    if (access(tmp, R_OK)==0)
-		iconPath = tmp;
-	}
-	if (!iconPath)
-	    free(tmp);
-    }
-    
-    if (iconPath) {
-	saveIconNameFor(iconPath, wm_instance, wm_class);
+        char *tmp;
 
-	free(iconPath);
+        tmp = wmalloc(strlen(path)+16);
+
+        if (scr->flags.supports_tiff) {
+            strcpy(tmp, path);
+            strcat(tmp, ".tiff");
+            if (access(tmp, R_OK)==0)
+                iconPath = tmp;
+        }
+        if (!path) {
+            strcpy(tmp, path);
+            strcat(tmp, ".xpm");
+            if (access(tmp, R_OK)==0)
+                iconPath = tmp;
+        }
+        if (!iconPath)
+            wfree(tmp);
+    }
+
+    if (iconPath) {
+        saveIconNameFor(iconPath, wm_instance, wm_class);
+
+        wfree(iconPath);
     }
 }
 
@@ -229,87 +235,103 @@ extractClientIcon(WAppIcon *icon)
 
     path = wIconStore(icon->icon);
     if (!path)
-	return False;
+        return False;
 
     saveIconNameFor(path, icon->wm_instance, icon->wm_class);
 
-    free(path);
+    wfree(path);
 
     return True;
 }
 
 
 WApplication*
-wApplicationCreate(WScreen *scr, Window main_window)
+wApplicationCreate(WWindow *wwin)
 {
+    WScreen *scr = wwin->screen_ptr;
+    Window main_window = wwin->main_window;
     WApplication *wapp;
     WWindow *leader;
 
     if (main_window==None || main_window==scr->root_win) {
 #ifdef DEBUG0
-	wwarning("trying to create application for %x",(unsigned)main_window);
+        wwarning("trying to create application for %x",(unsigned)main_window);
 #endif
-	return NULL;
+        return NULL;
     }
 
     {
-	Window root;
-	int foo;
-	unsigned int bar;
-	/* check if the window is valid */
-	if (!XGetGeometry(dpy, main_window, &root, &foo, &foo, &bar, &bar,
-			  &bar, &bar)) {
-	    return NULL;
-	}
+        Window root;
+        int foo;
+        unsigned int bar;
+        /* check if the window is valid */
+        if (!XGetGeometry(dpy, main_window, &root, &foo, &foo, &bar, &bar,
+                          &bar, &bar)) {
+            return NULL;
+        }
     }
-    
+
     wapp = wApplicationOf(main_window);
     if (wapp) {
-	wapp->refcount++;
-	return wapp;
+        wapp->refcount++;
+        if (wapp->app_icon && wapp->app_icon->docked &&
+            wapp->app_icon->relaunching && wapp->main_window_desc->fake_group) {
+            wDockFinishLaunch(wapp->app_icon->dock, wapp->app_icon);
+        }
+
+        return wapp;
     }
-    
+
     wapp = wmalloc(sizeof(WApplication));
     memset(wapp, 0, sizeof(WApplication));
 
     wapp->refcount = 1;
     wapp->last_focused = NULL;
-    
+
     wapp->last_workspace = 0;
-      
+
     wapp->main_window = main_window;
     wapp->main_window_desc = makeMainWindow(scr, main_window);
     if (!wapp->main_window_desc) {
-	free(wapp);
-	return NULL;
+        wfree(wapp);
+        return NULL;
     }
-    
+
+    wapp->main_window_desc->fake_group = wwin->fake_group;
+#ifdef NETWM_HINTS
+    wapp->main_window_desc->net_icon_image = RRetainImage(wwin->net_icon_image);
+#endif
+
     extractIcon(wapp->main_window_desc);
 
     leader = wWindowFor(main_window);
     if (leader) {
-	leader->main_window = main_window;
+        leader->main_window = main_window;
     }
     wapp->menu = wAppMenuGet(scr, main_window);
+#ifdef USER_MENU
+    if (!wapp->menu) wapp->menu = wUserMenuGet(scr, wapp->main_window_desc);
+#endif /* USER_MENU */
 
 
     /*
      * Set application wide attributes from the leader.
      */
-    wapp->flags.hidden = wapp->main_window_desc->window_flags.start_hidden;
-    
-    wapp->flags.emulated = wapp->main_window_desc->window_flags.emulate_appicon;
+    wapp->flags.hidden = WFLAGP(wapp->main_window_desc, start_hidden);
+
+    wapp->flags.emulated = WFLAGP(wapp->main_window_desc, emulate_appicon);
 
     /* application descriptor */
     XSaveContext(dpy, main_window, wAppWinContext, (XPointer)wapp);
 
-    if (!wapp->main_window_desc->window_flags.no_appicon) {
+    if (!WFLAGP(wapp->main_window_desc, no_appicon)) {
         wapp->app_icon = NULL;
         if (scr->last_dock)
             wapp->app_icon = findDockIconFor(scr->last_dock, main_window);
         /* check main dock if we did not find it in last dock */
-        if (!wapp->app_icon && scr->dock)
-	    wapp->app_icon = findDockIconFor(scr->dock, main_window);
+        if (!wapp->app_icon && scr->dock) {
+            wapp->app_icon = findDockIconFor(scr->dock, main_window);
+        }
         /* finally check clips */
         if (!wapp->app_icon) {
             int i;
@@ -321,7 +343,7 @@ wApplicationCreate(WScreen *scr, Window main_window)
                     break;
             }
         }
-                    
+
         if (wapp->app_icon) {
             WWindow *mainw = wapp->main_window_desc;
 
@@ -332,85 +354,78 @@ wApplicationCreate(WScreen *scr, Window main_window)
                 wapp->app_icon->icon->icon_win = mainw->wm_hints->icon_window;
             wAppIconPaint(wapp->app_icon);
         } else {
-	    wapp->app_icon = wAppIconCreate(wapp->main_window_desc);
-#ifdef REDUCE_APPICONS
-	/* This is so we get the appearance of invoking the app sitting
-	 * on the dock. -cls */
-	    if (wapp->app_icon) {
-		if (wapp->app_icon->docked && wapp->app_icon->num_apps == 1) {
-		    wapp->app_icon->launching = 0;
-		    wapp->app_icon->running = 1;
-		    wapp->app_icon->icon->force_paint = 1;
-		    wAppIconPaint(wapp->app_icon);
-		}
-	    }
-#endif
-	}
+            wapp->app_icon = wAppIconCreate(wapp->main_window_desc);
+        }
     } else {
-	wapp->app_icon = NULL;
-    }
-    
-    if (wapp->app_icon) {
-        wapp->app_icon->main_window = main_window;
-#ifdef WMSOUND
-        wSoundServerGrab(wapp->app_icon->wm_class, main_window);
-#endif
+        wapp->app_icon = NULL;
     }
 
-#ifndef REDUCE_APPICONS
-    if (wapp->app_icon && !wapp->app_icon->docked) {
-#else
-    if (wapp->app_icon && !wapp->app_icon->docked && wapp->app_icon->num_apps == 1) {
-#ifdef THIS_SUCKS
+    if (wapp->app_icon) {
+        wapp->app_icon->main_window = main_window;
     }
-#endif
-#endif
+
+    if (wapp->app_icon && !wapp->app_icon->docked) {
         WIcon *icon = wapp->app_icon->icon;
         WDock *clip = scr->workspaces[scr->current_workspace]->clip;
         int x=0, y=0;
 
         if (clip && clip->attract_icons && wDockFindFreeSlot(clip, &x, &y)) {
             wapp->app_icon->attracted = 1;
-            if (!clip->keep_attracted && !wapp->app_icon->icon->shadowed) {
-                wapp->app_icon->icon->shadowed = 1;
-                wapp->app_icon->icon->force_paint = 1;
-                /* We don't do an wAppIconPaint() here because it's in
-                 * wDockAttachIcon(). -Dan.
-                 */
+            if (!icon->shadowed) {
+                icon->shadowed = 1;
+                icon->force_paint = 1;
+                /* wAppIconPaint() is done in wDockAttachIcon() below */
             }
             wDockAttachIcon(clip, wapp->app_icon, x, y);
         } else {
-            PlaceIcon(scr, &x, &y);
-	    wAppIconMove(wapp->app_icon, x, y);
-	    wLowerFrame(icon->core);
+            PlaceIcon(scr, &x, &y, wGetHeadForWindow(wapp->main_window_desc));
+            wAppIconMove(wapp->app_icon, x, y);
+            wLowerFrame(icon->core);
         }
         if (!clip || !wapp->app_icon->attracted || !clip->collapsed)
-	    XMapWindow(dpy, icon->core->window);
+            XMapWindow(dpy, icon->core->window);
     }
 
     if (wPreferences.auto_arrange_icons && wapp->app_icon && !wapp->app_icon->attracted) {
-	wArrangeIcons(scr, True);
+        wArrangeIcons(scr, True);
     }
 
     if (wapp->app_icon) {
-	char *tmp;
+        char *tmp, *path;
+        struct stat dummy;
+        RImage *image;
 
-	/* if the displayed icon was supplied by the client, save the icon */
-	tmp = wDefaultGetIconFile(scr, wapp->app_icon->wm_instance,
-				  wapp->app_icon->wm_class, True);
-	if (!tmp)
-	    extractClientIcon(wapp->app_icon);
+        tmp = wDefaultGetIconFile(scr, wapp->app_icon->wm_instance,
+                                  wapp->app_icon->wm_class, True);
+
+        /* If the icon was saved by us from the client supplied icon, but is
+         * missing, recreate it. */
+        if (tmp && strstr(tmp, ".AppInfo/WindowMaker")!=NULL &&
+            stat(tmp, &dummy)!=0 && errno==ENOENT) {
+            wmessage(_("recreating missing icon '%s'"), tmp);
+            path = wIconStore(wapp->app_icon->icon);
+            if (path) {
+                wfree(path);
+            }
+            image = wDefaultGetImage(scr, wapp->app_icon->wm_instance,
+                                     wapp->app_icon->wm_class);
+            if (image) {
+                wIconChangeImage(wapp->app_icon->icon, image);
+                wAppIconPaint(wapp->app_icon);
+                /* TODO:
+                 * wIconChangeImage() should be rewriten to use retain/release
+                 * The way it is now is too confusing about where the icon is
+                 * finally released.  -Dan */
+                /* --this is wrong at the moment-- RReleaseImage(image);*/
+            }
+        }
+
+        /* if the displayed icon was supplied by the client, save the icon */
+        if (!tmp)
+            extractClientIcon(wapp->app_icon);
     }
-    
-    wapp->prev = NULL; 
-    wapp->next = scr->wapp_list;
-    if (scr->wapp_list)
-	scr->wapp_list->prev = wapp;
-    scr->wapp_list = wapp;
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_APPSTART);
-#endif
+    wSoundPlay(WSOUND_APPSTART);
 
 #ifdef DEBUG
     printf("Created application for %x\n", (unsigned)main_window);
@@ -425,23 +440,17 @@ wApplicationDestroy(WApplication *wapp)
     Window main_window;
     WWindow *wwin;
     WScreen *scr;
-#ifdef REDUCE_APPICONS
-    unsigned int napps;
-#endif
 
     if (!wapp)
-      return;
-    
+        return;
+
     wapp->refcount--;
     if (wapp->refcount>0)
-      return;
+        return;
 
 
     scr = wapp->main_window_desc->screen_ptr;
     main_window = wapp->main_window;
-#ifdef REDUCE_APPICONS
-    napps = wAppIconReduceAppCount(wapp);
-#endif
 
     if (wapp == scr->wapp_list) {
         if (wapp->next)
@@ -457,55 +466,35 @@ wApplicationDestroy(WApplication *wapp)
     XDeleteContext(dpy, wapp->main_window, wAppWinContext);
     wAppMenuDestroy(wapp->menu);
     if (wapp->app_icon) {
-        if (wapp->app_icon->docked
-            && (!wapp->app_icon->attracted || 
-		wapp->app_icon->dock->keep_attracted)) {
-#ifdef REDUCE_APPICONS
-	    if (napps == 0) {
-#endif
-		wapp->app_icon->running = 0;
-		/* since we keep it, we don't care if it was attracted or not */
-		wapp->app_icon->attracted = 0;
-		wapp->app_icon->icon->shadowed = 0;
-		wapp->app_icon->main_window = None;
-		wapp->app_icon->pid = 0;
-		wapp->app_icon->icon->owner = NULL;
-		wapp->app_icon->icon->icon_win = None;
-		wapp->app_icon->icon->force_paint = 1;
-		wAppIconPaint(wapp->app_icon);
-#ifdef REDUCE_APPICONS
-	    }
-#endif
+        if (wapp->app_icon->docked && !wapp->app_icon->attracted) {
+            wapp->app_icon->running = 0;
+            /* since we keep it, we don't care if it was attracted or not */
+            wapp->app_icon->attracted = 0;
+            wapp->app_icon->icon->shadowed = 0;
+            wapp->app_icon->main_window = None;
+            wapp->app_icon->pid = 0;
+            wapp->app_icon->icon->owner = NULL;
+            wapp->app_icon->icon->icon_win = None;
+            wapp->app_icon->icon->force_paint = 1;
+            wAppIconPaint(wapp->app_icon);
         } else if (wapp->app_icon->docked) {
-#ifdef REDUCE_APPICONS
-	    if (napps == 0)  {
-#endif
-		wapp->app_icon->running = 0;
-		wDockDetach(wapp->app_icon->dock, wapp->app_icon);
-#ifdef REDUCE_APPICONS
-	    }
-#endif 
+            wapp->app_icon->running = 0;
+            wDockDetach(wapp->app_icon->dock, wapp->app_icon);
         } else {
-#ifdef REDUCE_APPICONS
-    	    if (napps == 0) {
-#endif 
-		wAppIconDestroy(wapp->app_icon);
-#ifdef REDUCE_APPICONS
-    	    }
-#endif
-	}
+            wAppIconDestroy(wapp->app_icon);
+        }
     }
     wwin = wWindowFor(wapp->main_window_desc->client_win);
 
     wWindowDestroy(wapp->main_window_desc);
     if (wwin) {
-    	/* undelete client window context that was deleted in
-     	 * wWindowDestroy */
-    	XSaveContext(dpy, wwin->client_win, wWinContext, 
-		     (XPointer)&wwin->client_descriptor);
+        /* undelete client window context that was deleted in
+         * wWindowDestroy */
+        XSaveContext(dpy, wwin->client_win, wWinContext,
+                     (XPointer)&wwin->client_descriptor);
     }
-    free(wapp);
-    
+    wfree(wapp);
+
 #ifdef DEBUG
     printf("Destroyed application for %x\n", (unsigned)main_window);
 #endif
@@ -513,9 +502,8 @@ wApplicationDestroy(WApplication *wapp)
         wArrangeIcons(scr, True);
     }
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_APPEXIT);
-#endif
+    wSoundPlay(WSOUND_APPEXIT);
 }
 
 
+
